// lcd_test.c
#include <pic18.h>
#include <pic18fx490.h>
#include "lcd.h"
#include "lcd_test.h"
#include "timers.h"
#include "usart.h"


#define DELAY_TIME 			300
#define DT 					150
#define	DELAY_TIME_LONG		2000
#define	BIT5	0x20


volatile unsigned char mask_port_a 	= 0x33; 
volatile unsigned char mask_port_b 	= 0x20&(~BIT5); // pull up BIT5
volatile unsigned char mask_port_c 	= 0x38;

volatile unsigned char temp_porta 	= 0x0;
volatile unsigned char temp_portb 	= 0x0;
volatile unsigned char temp_portc 	= 0x0; 

unsigned char	err_vcc_a	= 0;
unsigned char	err_vcc_b	= 0;
unsigned char	err_vcc_c	= 0;
unsigned char	err_gnd_a	= 0;
unsigned char	err_gnd_b	= 0;
unsigned char	err_gnd_c	= 0;
unsigned char	err_pin_a	= 10;
unsigned char	err_pin_b	= 10;
unsigned char	err_pin_c	= 10;

unsigned int 	i		= 0;

// simple delay ~ 1 mS
void Delay(unsigned long a) { 
	 a *= 370;
	while (--a!=0); 
}




void Stop(void) {

	// Init Usart1
	InitUsart1();

	// InitTimer1
	InitTimer1();

	if(err_vcc_a) {
		Usart1WriteText("\n\rPin to VCC at PortA\0");
	}

	if(err_vcc_b) {
		Usart1WriteText("\n\rPin to VCC at PortB\0");
	}	

	if(err_vcc_c) {
		Usart1WriteText("\n\rPin to VCC at PortC\0");	
	}

	if(err_gnd_a) {
		Usart1WriteText("\n\rPin to GND at PortA\0");	
	}

	if(err_gnd_b) {
		Usart1WriteText("\n\rPin to GND at PortB\0");		
	}

	if(err_gnd_c) {
		Usart1WriteText("\n\rPin to GND at PortC\0");		
	}

	if(err_pin_a!=10) {
		Usart1WriteText("\n\rPin error -> ");			
		Usart1WriteChar(err_pin_a+48);
		Usart1WriteText(" at PortA\0");			
	}

	if(err_pin_b!=10) {
		Usart1WriteText("\n\rPin error -> ");			
		Usart1WriteChar(err_pin_b+48);
		Usart1WriteText(" at PortB\0");				
	}

	if(err_pin_c!=10) {
		Usart1WriteText("\n\rPin error -> ");			
		Usart1WriteChar(err_pin_c+48);
		Usart1WriteText(" at PortC\0");				
	}

	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	while(1);
}



void TestExt(void) {


	// TEST EXTENSION PORT

	// Init Usart1
	InitUsart1();

	// InitTimer1
	InitTimer1();

	// Write to Usart1
	Usart1WriteText("\n\rTest Extension port begin.... ");


	// Check for GND ======================================
	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	//pull up - output
	TRISB &= ~BIT5;
	//pull up - high
	PORTB |= BIT5;

	Delay(10);

	temp_porta 	= PORTA;
	temp_porta 	|= (~mask_port_a); 
	if((temp_porta) != (0xFF)) {
		err_gnd_a = 1;
		Stop();
	} 


	temp_portb 	= PORTB;
	temp_portb 	|=(~mask_port_b); 
	if((temp_portb) != (0xFF)) {
		err_gnd_b = 1;
		Stop();
	} 

	temp_portc 	= PORTC;
	temp_portc 	|=(~mask_port_c); 
	if((temp_portc) != (0xFF)) {
		err_gnd_c = 1;
		Stop();
	} 


	// Check for VCC =============================

	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	//pull up - output
	TRISB &= ~BIT5;
	//pull up - low
	PORTB &= ~BIT5;

	Delay(10);

	temp_porta 	= PORTA;
	temp_porta 	&= mask_port_a; 
	if((temp_porta) != (0x0)) {
		err_vcc_a = 1;
		Stop();
	} 

 
	temp_portb 	= PORTB;
	temp_portb 	&= mask_port_b; 
	if((temp_portb) != (0x0)) {
		err_vcc_b = 1;
		Stop();
	} 


	temp_portc 	= PORTC;
	temp_portc 	&= mask_port_c; 
	if((temp_portc) != (0x0)) {
		err_vcc_c = 1;
		Stop();
	} 



	// Running zero =========================================
	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	// PortA
	//pull up - output
	TRISB &= ~BIT5;
	//pull up - high
	PORTB |= BIT5;
	
	Delay(10);

	for(i=0; i<8; i++) {
	
		// this port is not tested
		if(!((mask_port_a)&(1<<i))) continue;
	
		TRISA = ~(1<<i);
		PORTA = ~(1<<i);
		
		Delay(10);

		// check for other zero at PortA
		temp_porta 	= PORTA;
		temp_porta 	|= (~mask_port_a); 
		if((temp_porta) != (0xFF&(~(1<<i)))) {
			err_pin_a = i;
			Stop();
		} 

		// check for other zero at PortB	 
		temp_portb 	= PORTB;
		temp_portb 	|=(~mask_port_b); 
		if((temp_portb) != (0xFF)) {
			err_pin_b = i;
			Stop();
		} 
	
		// check for other zero at PortC
		temp_portc 	= PORTC;
		temp_portc 	|=(~mask_port_c); 
		if((temp_portc) != (0xFF)) {
			err_pin_c = i;
			Stop();
		} 
	
	}


	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	// PortB
	//pull up - output
	TRISB &= ~BIT5;
	//pull up - high
	PORTB |= BIT5;

	Delay(10);

	for(i=0; i<8; i++) {
	
		// this port is not tested
		if(!((mask_port_b)&(1<<i))) continue;
	
		TRISB = ~((1<<i)|BIT5);
		PORTB = ~(1<<i);
		
		Delay(10);

		// check for other zero at PortA
		temp_porta 	= PORTA;
		temp_porta 	|= (~mask_port_a); 
		if((temp_porta) != (0xFF)) {
			err_pin_a = i;
			Stop();
		} 

		// check for other zero at PortB	 
		temp_portb 	= PORTB;
		temp_portb 	|=(~mask_port_b); 
		if((temp_portb) != (0xFF&(~(1<<i)))) {
			err_pin_b = i;
			Stop();
		} 
	
		// check for other zero at PortC
		temp_portc 	= PORTC;
		temp_portc 	|=(~mask_port_c); 
		if((temp_portc) != (0xFF)) {
			err_pin_c = i;
			Stop();
		} 
	
	}

	// all as input
	ADCON1 	= 0x0F;
	CMCON	= 0x07;
	TRISA 	= 0xFF;
	TRISC 	= 0xFF;
	TRISB 	= 0xFF;

	// PortC
	//pull up - output
	TRISB &= ~BIT5;
	//pull up - high
	PORTB |= BIT5;

	Delay(10);

	for(i=0; i<8; i++) {
	
		// this port is not tested
		if(!((mask_port_c)&(1<<i))) continue;
	
		TRISC = ~(1<<i);
		PORTC = ~(1<<i);
		
		Delay(10);
	
		// check for other zero at PortA
		temp_porta 	= PORTA;
		temp_porta 	|= (~mask_port_a); 
		if((temp_porta) != (0xFF)) {
			err_pin_a = i;
			Stop();
		} 

		// check for other zero at PortB	 
		temp_portb 	= PORTB;
		temp_portb 	|=(~mask_port_b); 
		if((temp_portb) != (0xFF)) {
			err_pin_b = i;	
			Stop();
		} 
	
		// check for other zero at PortC
		temp_portc 	= PORTC;
		temp_portc 	|=(~mask_port_c); 
		if((temp_portc) != (0xFF&(~(1<<i)))) {
			err_pin_c = i;
			Stop();
		} 
	
	}


	// Init Usart1
	InitUsart1();

	// InitTimer1
	InitTimer1();

	// Write to Usart1
	Usart1WriteText("Test OK!!!\0");

	// END TEST EXTENSION PORT

}


// test lcd segment
void TestLCD(void) {

	char i = 0;

	LCD_light_all();
	Delay(DELAY_TIME_LONG);
	LCD_clear_all();
	Delay(DELAY_TIME_LONG/2);
	
	// progress
	_progress_off; _progress_start;		Delay(DT); 
	_progress_off; _progress_1; 		Delay(DT); 
	_progress_off; _progress_2; 		Delay(DT); 
	_progress_off; _progress_3; 		Delay(DT); 
	_progress_off; _progress_4; 		Delay(DT); 
	_progress_off; _progress_5; 		Delay(DT); 
	_progress_off; _progress_6; 		Delay(DT); 
	_progress_off; _progress_7; 		Delay(DT); 
	_progress_off; _progress_8; 		Delay(DT); 
	_progress_off; _progress_9; 		Delay(DT); 
	_progress_off; _progress_10; 		Delay(DT); 
	_progress_off; _progress_end;  		Delay(DT); 	
	_progress_off;

	// arrow
	_arr_off; 	_arr_l;  	Delay(DT);
	_arr_off; 	_arr_r;  	Delay(DT);
	_arr_off; 	_arr_d;  	Delay(DT);
	_arr_off; 	_arr_u;  	Delay(DT);
	_arr_off;

	// small digit
	clk_off; _a1; Delay(DT); clk_off; _b1; Delay(DT); clk_off; _c1; Delay(DT); 
	clk_off; _d1; Delay(DT); clk_off; _e1; Delay(DT); clk_off; _f1; Delay(DT);
	clk_off; _g1; Delay(DT); clk_off;

	clk_off; _a2; Delay(DT); clk_off; _b2; Delay(DT); clk_off; _c2; Delay(DT); 
	clk_off; _d2; Delay(DT); clk_off; _e2; Delay(DT); clk_off; _f2; Delay(DT);
	clk_off; _g2; Delay(DT); clk_off;

	clk_off; _a3; Delay(DT); clk_off; _b3; Delay(DT); clk_off; _c3; Delay(DT); 
	clk_off; _d3; Delay(DT); clk_off; _e3; Delay(DT); clk_off; _f3; Delay(DT);
	clk_off; _g3; Delay(DT); clk_off;

	clk_off; _a4; Delay(DT); clk_off; _b4; Delay(DT); clk_off; _c4; Delay(DT); 
	clk_off; _d4; Delay(DT); clk_off; _e4; Delay(DT); clk_off; _f4; Delay(DT);
	clk_off; _g4; Delay(DT); clk_off;

	// dot clock
	_dot_off; 	_dot1;	Delay(DT);
	_dot_off; 	_dot2;	Delay(DT);
	_dot_off; 	_dot3;	Delay(DT);
	_dot_off;	_ddot;	Delay(DT);
	_dot_off;

	// sing and battery
	sing1_off; _u;  Delay(DT); 
	sing1_off; _m;  Delay(DT); 
	sing1_off; _H;  Delay(DT); 
	sing1_off; _F;  Delay(DT);  
	sing1_off;

	lbat_off;  	_lbat_start_end;	Delay(DT); 
	lbat_off; 	_lbat_1				Delay(DT); 
	lbat_off;	_lbat_2				Delay(DT); 
	lbat_off;	_lbat_3				Delay(DT); 
	lbat_off;
	

	// plus - minus - olimex
	sing2_off;	_OLIMEX;	Delay(DT);
	sing2_off;	_MINUS;		Delay(DT);
	sing2_off;	_PLUS;		Delay(DT);
	sing2_off;


	// digit
	clr1; _A1; Delay(DT); clr1; _B1; Delay(DT); clr1; _C1; Delay(DT); clr1; _D1; Delay(DT); 
	clr1; _E1; Delay(DT); clr1; _F1; Delay(DT); clr1; _G1; Delay(DT); clr1; _H1; Delay(DT); 
	clr1; _J1; Delay(DT); clr1; _K1; Delay(DT); clr1; _M1; Delay(DT); clr1; _N1; Delay(DT); 
	clr1; _P1; Delay(DT); clr1; _Q1; Delay(DT); _all_1_off;

	clr2; _A2; Delay(DT); clr2; _B2; Delay(DT); clr2; _C2; Delay(DT); clr2; _D2; Delay(DT); 
	clr2; _E2; Delay(DT); clr2; _F2; Delay(DT); clr2; _G2; Delay(DT); clr2; _H2; Delay(DT); 
	clr2; _J2; Delay(DT); clr2; _K2; Delay(DT); clr2; _M2; Delay(DT); clr2; _N2; Delay(DT); 
	clr2; _P2; Delay(DT); clr2; _Q2; Delay(DT); _all_2_off;

	clr3; _A3; Delay(DT); clr3; _B3; Delay(DT); clr3; _C3; Delay(DT); clr3; _D3; Delay(DT); 
	clr3; _E3; Delay(DT); clr3; _F3; Delay(DT); clr3; _G3; Delay(DT); clr3; _H3; Delay(DT); 
	clr3; _J3; Delay(DT); clr3; _K3; Delay(DT); clr3; _M3; Delay(DT); clr3; _N3; Delay(DT); 
	clr3; _P3; Delay(DT); clr3; _Q3; Delay(DT); _all_3_off;

	clr4; _A4; Delay(DT); clr4; _B4; Delay(DT); clr4; _C4; Delay(DT); clr4; _D4; Delay(DT); 
	clr4; _E4; Delay(DT); clr4; _F4; Delay(DT); clr4; _G4; Delay(DT); clr4; _H4; Delay(DT); 
	clr4; _J4; Delay(DT); clr4; _K4; Delay(DT); clr4; _M4; Delay(DT); clr4; _N4; Delay(DT); 
	clr4; _P4; Delay(DT); clr4; _Q4; Delay(DT); _all_4_off;

	clr5; _A5; Delay(DT); clr5; _B5; Delay(DT); clr5; _C5; Delay(DT); clr5; _D5; Delay(DT); 
	clr5; _E5; Delay(DT); clr5; _F5; Delay(DT); clr5; _G5; Delay(DT); clr5; _H5; Delay(DT); 
	clr5; _J5; Delay(DT); clr5; _K5; Delay(DT); clr5; _M5; Delay(DT); clr5; _N5; Delay(DT); 
	clr5; _P5; Delay(DT); clr5; _Q5; Delay(DT); _all_5_off;

	clr6; _A6; Delay(DT); clr6; _B6; Delay(DT); clr6; _C6; Delay(DT); clr6; _D6; Delay(DT); 
	clr6; _E6; Delay(DT); clr6; _F6; Delay(DT); clr6; _G6; Delay(DT); clr6; _H6; Delay(DT); 
	clr6; _J6; Delay(DT); clr6; _K6; Delay(DT); clr6; _M6; Delay(DT); clr6; _N6; Delay(DT); 
	clr6; _P6; Delay(DT); clr6; _Q6; Delay(DT); _all_6_off;

	clr7; _A7; Delay(DT); clr7; _B7; Delay(DT); clr7; _C7; Delay(DT); clr7; _D7; Delay(DT); 
	clr7; _E7; Delay(DT); clr7; _F7; Delay(DT); clr7; _G7; Delay(DT); clr7; _H7; Delay(DT); 
	clr7; _J7; Delay(DT); clr7; _K7; Delay(DT); clr7; _M7; Delay(DT); clr7; _N7; Delay(DT); 
	clr7; _P7; Delay(DT); clr7; _Q7; Delay(DT); _all_7_off;

	// dots, ddots
	DOT_off; 	_DOT1;	Delay(DT);
	DOT_off; 	_DOT2;	Delay(DT);
	DOT_off; 	_DOT3;	Delay(DT);
	DOT_off; 	_DOT4;	Delay(DT);
	DOT_off; 	_DOT5;	Delay(DT);
	DOT_off; 	_DOT6;	Delay(DT);
	DOT_off; 

	DDOT_off; 	_DDOT1;	Delay(DT);
	DDOT_off; 	_DDOT2;	Delay(DT);
	DDOT_off; 	_DDOT3;	Delay(DT);
	DDOT_off;	_DDOT4;	Delay(DT);	
	DDOT_off;	_DDOT5;	Delay(DT);
	DDOT_off;

}





